package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.dto.ApplicationResourceDTO;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.erp.operation.api.dto.ApiFieldDTO;
import com.gic.erp.operation.api.service.ApiFieldApiService;
import com.gic.gateway.sdk.anno.GatewayExporter;
import com.gic.gateway.sdk.anno.GatewayParams;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.constant.StoreLogReasonEnum;
import com.gic.store.dto.*;
import com.gic.store.dto.open.BusinessTimeOpenDTO;
import com.gic.store.dto.open.ClerkDTO;
import com.gic.store.dto.open.ClerkDetailDTO;
import com.gic.store.dto.open.StoreOpenDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabStore;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service("storeOpenApiService")
@GatewayExporter
public class StoreOpenApiServiceImpl implements StoreOpenApiService {
    private static final Logger LOGGER = LogManager.getLogger(StoreOpenApiServiceImpl.class);
    @Autowired
    private ApiFieldApiService apiFieldApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private ClerkApiService clerkApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private ProvincesApiService provincesApiService;
    @Autowired
    private StoreFieldService storeFieldService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private ResourceApiService resourceApiService;

    @Override
    public ServiceResponse<String> getApiFieldJson(Integer enterpriseId, Integer regionId) {
        ServiceResponse<List<ApiFieldDTO>> result = apiFieldApiService.listApiField(2, enterpriseId,
                Long.parseLong(regionId.toString()));
        if (result.isSuccess()) {
            List<ApiFieldDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return ServiceResponse.success(JSON.toJSONString(list));
            }
        }
        return ServiceResponse.success();
    }

    public Map<String, String> getApiFieldMap(Integer enterpriseId, Integer regionId) {
        ServiceResponse<List<ApiFieldDTO>> result = apiFieldApiService.listApiField(2, enterpriseId, Long.parseLong(regionId.toString()));
        if (result.isSuccess()) {
            List<ApiFieldDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return list.stream().collect(Collectors.toMap(ApiFieldDTO::getFieldCode, ApiFieldDTO::getGicFieldName));
            }
        }
        return null;
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeCode", "regionId"})
    public ServiceResponse<String> getStoreDetail(String appId, Integer enterpriseId, String storeCode, Integer regionId) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (StringUtils.isBlank(storeCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (regionId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        if (!hasAuth(appId, enterpriseId, storeCode, regionId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
        }
        return ServiceResponse.success(JSON.toJSONString(store));
    }

    @Override
    @GatewayParams(query = {"appId","enterpriseId"}, body = {"storeOpenDTO"})
    public ServiceResponse<String> addStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO) {
        LOGGER.info("对外API新增门店参数：{}", JSON.toJSONString(storeOpenDTO));
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        StoreDTO storeDTO = EntityUtil.changeEntityByJSON(StoreDTO.class, storeOpenDTO);
        storeDTO.setEnterpriseId(enterpriseId);
        if (StringUtils.isBlank(storeOpenDTO.getStoreName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店名称不能为空");
        }
        if (StringUtils.isBlank(storeOpenDTO.getStoreCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (storeOpenDTO.getRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        if (StringUtils.isBlank(storeOpenDTO.getBrand())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不能为空");
        }
        // 校验权限
        if (!hasAuth(appId, enterpriseId, storeOpenDTO.getStoreCode(), storeOpenDTO.getRegionId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }

        // 校验门店类型
        if (storeOpenDTO.getErpType() != null) {
            ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreType(enterpriseId);
            if (serviceResponse.isSuccess()) {
                List<StoreDictDTO> list = serviceResponse.getResult();
                Integer erpType = storeOpenDTO.getErpType();
                String storeTypeTmp = erpType.toString();
                for (StoreDictDTO storeDictDTO : list) {
                    if (storeDictDTO.getValue().equals(storeTypeTmp)) {
                        storeDTO.setStoreType(Integer.valueOf(storeDictDTO.getValue()));
                        break;
                    }
                }
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "调用门店类型接口失败");
            }
            // 没有匹配上
            if (storeDTO.getStoreType() == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "门店类型不存在");
            }
        }

        // 校验门店状态
        if (storeOpenDTO.getErpStatus() != null) {
            ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreStatus(enterpriseId);
            if (serviceResponse.isSuccess()) {
                List<StoreDictDTO> list = serviceResponse.getResult();
                Integer erpStatus = storeOpenDTO.getErpStatus();
                String storeStatusType = erpStatus.toString();
                for (StoreDictDTO storeDictDTO : list) {
                    if (storeDictDTO.getValue().equals(storeStatusType)) {
                        storeDTO.setStoreType(Integer.valueOf(storeDictDTO.getValue()));
                        break;
                    }
                }
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "调用门店状态接口失败");
            }
            // 没有匹配上
            if (storeDTO.getStoreType() == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "门店状态不存在");
            }
        }


        if (StringUtils.isNotBlank(storeOpenDTO.getGroupName())) {
            TabStoreGroup group = storeGroupService.getStoreGroupByName(storeOpenDTO.getGroupName(), enterpriseId);
            if (group == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组不存在");
            } else {
                storeDTO.setStoreGroupId(group.getStoreGroupId());
            }
        }

        TabStoreBrand brand = storeBrandService.getByStoreBrandName(enterpriseId, storeOpenDTO.getBrand());
        if (brand == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不存在");
        }
        storeDTO.setBrandIds(brand.getStoreBrandId().toString());

        if(StringUtils.isNotBlank(storeOpenDTO.getProvince()) && StringUtils.isNotBlank(storeOpenDTO.getCity()) && StringUtils.isNotBlank(storeOpenDTO.getCounty())) {
            ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
            ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
            ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
            List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
            String finalProvince = storeOpenDTO.getProvince();
            ProvinceDTO provinceDTO = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), finalProvince)).findFirst().orElse(null);
            List<CityDTO> cityDTOList = cityResponse.getResult();
            String finalCity = storeOpenDTO.getCity();
            CityDTO cityDTO = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), finalCity)).findFirst().orElse(null);
            List<CountyDTO> countyDTOList = countyResponse.getResult();
            String finalCounty = storeOpenDTO.getCounty();
            CountyDTO countyDTO = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), finalCounty)).findFirst().orElse(null);
            if (countyDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的区县信息有误");
            }
            if (cityDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的市信息有误");
            }
            if (provinceDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的省信息有误");
            }
            storeDTO.setProvinceId(provinceDTO.getProvinceId());
            storeDTO.setCityId(cityDTO.getCityId());
            storeDTO.setAreaId(countyDTO.getCountyId());
            storeDTO.setProvinces(provinceDTO.getProvinceName() + "/" + cityDTO.getCityName() + "/" + countyDTO.getCountyName());
        }
        // 营业时间
        if (StringUtils.isNotBlank(storeOpenDTO.getBusinessTime())) {
            try {
                List<BusinessTimeOpenDTO> businessTimeList = JSON.parseArray(storeOpenDTO.getBusinessTime(), BusinessTimeOpenDTO.class);
                List<StoreBusinessTimeDTO> businessTimeResultList = new ArrayList<>();
                for (BusinessTimeOpenDTO businessTimeOpenDTO : businessTimeList) {
                    StoreBusinessTimeDTO businessTimeDTO = new StoreBusinessTimeDTO();
                    businessTimeDTO.setWeekday(businessTimeOpenDTO.getWeekday());
                    businessTimeDTO.setOpenTime(DateUtil.strToDate(businessTimeOpenDTO.getOpenTime(), DateUtil.FORMAT_MINUTE_TIME));
                    businessTimeDTO.setCloseTime(DateUtil.strToDate(businessTimeOpenDTO.getCloseTime(), DateUtil.FORMAT_MINUTE_TIME));
                    businessTimeResultList.add(businessTimeDTO);
                }
                storeDTO.setBusinessTimeList(businessTimeResultList);
            } catch (Exception e) {
                LOGGER.info("输入的营业时间有误:{}", storeOpenDTO.getBusinessTime(), e);
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的营业时间有误");
            }
        }
        // 门店照片
        if (StringUtils.isNotBlank(storeOpenDTO.getPhotos())) {
            List<String> photoList = JSON.parseArray(storeOpenDTO.getPhotos(), String.class);
            List<StorePhotoDTO> photoResultList = new ArrayList<>();
            for (String imageUrl : photoList) {
                StorePhotoDTO storePhotoDTO = new StorePhotoDTO();
                storePhotoDTO.setImageUrl(imageUrl);
                photoResultList.add(storePhotoDTO);
            }
            storeDTO.setPhotoList(photoResultList);
        }

        // 自定义属性
        List<StoreFieldDTO> fieldList = storeFieldService.listStoreFieldByRegionId(storeOpenDTO.getRegionId());
        String customField = storeOpenDTO.getCustomField();
        if (StringUtils.isBlank(customField)) {
            customField = "[]";
        }
        List<JSONObject> customFieldList = JSON.parseArray(customField, JSONObject.class);
        if (CollectionUtils.isNotEmpty(customFieldList)) {

            Map<String, String> apiFieldMap = getApiFieldMap(enterpriseId, storeOpenDTO.getRegionId());
            if (apiFieldMap == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "获取gic端和对外API之间门店自定义字段的匹配数据失败");
            }
            for (JSONObject jsonObject : customFieldList) {
                String fieldCode = jsonObject.getString("fieldCode");
                String fieldName = apiFieldMap.get(fieldCode);
                jsonObject.put("fieldName", fieldName);
            }

//        logger.info("门店域的ID：{}, 自定义字段数据:{}", regionId, JSON.toJSONString(fieldList));
            if (CollectionUtils.isNotEmpty(fieldList)) {
                List<StoreExtendDTO> extendDTOList = new ArrayList<>();
                for (int i = 0; i < fieldList.size(); i++) {
                    StoreExtendDTO storeExtendDTO = new StoreExtendDTO();
                    storeExtendDTO.setStoreFieldId(fieldList.get(i).getStoreFieldId());
                    storeExtendDTO.setStoreFieldType(fieldList.get(i).getStoreFieldType());
                    storeExtendDTO.setStoreFieldName(fieldList.get(i).getStoreFieldName());
                    storeExtendDTO.setStoreFieldCode(fieldList.get(i).getStoreFieldCode());
                    for (JSONObject storeFieldObject : customFieldList) {
                        if (fieldList.get(i).getStoreFieldName().equals(storeFieldObject.getString("fieldName"))) {
                            storeExtendDTO.setValue(storeFieldObject.getString("fieldValue"));
                            break;
                        }
                    }
                    storeExtendDTO.setSort(fieldList.get(i).getSort());
                    // 验证
                    ServiceResponse fieldResponse = storeApiService.validateFieldExceptRepeat(storeExtendDTO.getValue(), fieldList.get(i));
                    if (!fieldResponse.isSuccess()) {
                        return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), fieldResponse.getMessage());
                    }
                    extendDTOList.add(storeExtendDTO);
                }
                storeDTO.setStoreExtendList(extendDTOList);
            } else {
                storeDTO.setStoreExtendList(Collections.emptyList());
            }
        }

        storeDTO.setReason(StoreLogReasonEnum.EXTERNAL.getCode());
        storeDTO.setCreateType(CreateTypeEnum.API.getCode());
        if (storeDTO.getStoreExtendList() == null) {
            storeDTO.setStoreExtendList(Collections.emptyList());
        }
        // 保存门店
//        return null;
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.saveOrUpdate(storeDTO);
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        } else {
            return EnterpriseServiceResponse.success();
        }
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"}, body = {"storeOpenDTO"})
    public ServiceResponse<String> editStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO) {
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (StringUtils.isBlank(storeOpenDTO.getStoreCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (storeOpenDTO.getRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        TabStore oldStore = this.storeService.getByStoreCodeAndRegionId(enterpriseId, storeOpenDTO.getRegionId(), storeOpenDTO.getStoreCode());
        if (oldStore != null) {
            storeOpenDTO.setStoreId(oldStore.getStoreId());
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店不存在");
        }

        // 校验权限
        if (!hasAuth(appId, enterpriseId, storeOpenDTO.getStoreCode(), storeOpenDTO.getRegionId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }

        return this.addStore(appId, enterpriseId, storeOpenDTO);
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "regionId", "pageNum", "pageSize"})
    public ServiceResponse<String> pageStore(String appId, Integer enterpriseId, Integer regionId, Integer pageNum, Integer pageSize) {
        if (enterpriseId == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户id不能为空");
        }
        if (regionId == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setRegionId(regionId);

        // 校验权限
        if (!hasAuth(appId, enterpriseId, null, regionId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        String storeIds = getStoreIds(appId, enterpriseId, regionId).toString();
        if (StringUtils.isBlank(storeIds)) {
            return ServiceResponse.success(JSON.toJSONString(new Page()));
        }
        storeSearchDTO.setStoreIds(storeIds);

        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }

        ServiceResponse<Page<StoreDTO>> serviceResponse = this.storeApiService.listStore(storeSearchDTO, pageNum, pageSize);
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        return ServiceResponse.success(JSON.toJSONString(page));
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeCode", "clerkCode", "regionId"})
    public ServiceResponse<String> getClerkDetail(String appId, Integer enterpriseId, String storeCode, String clerkCode, Integer regionId) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (StringUtils.isBlank(storeCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (regionId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
        }
        if (!hasAuth(appId, enterpriseId, storeCode, regionId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        Integer storeInfoId = store.getStoreInfoId();
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        if (clerk == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购代码错误，无此导购");
        }
        ClerkDetailDTO result = EntityUtil.changeEntityNew(ClerkDetailDTO.class, clerk);
        result.setStoreCode(storeCode);
        return ServiceResponse.success(JSON.toJSONString(result));
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId"}, body = {"clerkInfo"})
    public ServiceResponse<Void> addClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo) {
        LOGGER.info("对外API新增导购参数:{}", JSON.toJSONString(clerkInfo));
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (StringUtils.isBlank(clerkInfo.getStoreCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (clerkInfo.getRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, clerkInfo.getRegionId(), clerkInfo.getStoreCode());
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
        }
        Integer storeInfoId = store.getStoreInfoId();

        if (!hasAuth(appId, enterpriseId, clerkInfo.getStoreCode(), clerkInfo.getRegionId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称不能为空");
        }
        Integer gender = clerkInfo.getClerkGender();
        if (gender == null || (gender.intValue() != 1 && gender.intValue() != 0)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购性别值非法");
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号不能为空");
        }
        if (phone.trim().length() != 11) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号非法");
        }

        com.gic.store.dto.ClerkDTO dto = EntityUtil.changeEntityNew(com.gic.store.dto.ClerkDTO.class, clerkInfo);
        dto.setStoreInfoId(storeInfoId);
        dto.setNationcode(Constants.NATION_CODE);
        dto.setEnterpriseId(enterpriseId);
        //接口会验证
        // 权限 、 手机号重复 、 导购名称重复 、导购code重复
        return clerkApiService.saveOrUpdate(dto);
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId"}, body = {"clerkInfo"})
    public ServiceResponse<Void> editClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (StringUtils.isBlank(clerkInfo.getStoreCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (clerkInfo.getRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, clerkInfo.getRegionId(), clerkInfo.getStoreCode());
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
        }
        Integer storeInfoId = store.getStoreInfoId();

        if (!hasAuth(appId, enterpriseId, clerkInfo.getStoreCode(), clerkInfo.getRegionId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkInfo.getClerkCode(), storeInfoId);
        if (clerk == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code错误，无此导购");
        }

        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称不能为空");
        }
        Integer gender = clerkInfo.getClerkGender();
        if (gender == null || (gender.intValue() != 1 && gender.intValue() != 0)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购性别值非法");
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号不能为空");
        }
        if (phone.trim().length() != 11) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号非法");
        }

        com.gic.store.dto.ClerkDTO dto = EntityUtil.changeEntityNew(com.gic.store.dto.ClerkDTO.class, clerkInfo);
        dto.setStoreInfoId(storeInfoId);
        dto.setNationcode(Constants.NATION_CODE);
        dto.setEnterpriseId(enterpriseId);
        dto.setClerkId(clerk.getClerkId());
        //接口会验证
        // 权限 、 手机号重复 、 导购名称重复 、导购code重复
        return clerkApiService.saveOrUpdate(dto);
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeCode", "regionId", "pageNum", "pageSize"})
    public ServiceResponse<String> pageClerk(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer pageNum, Integer pageSize) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (regionId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        if (!hasAuth(appId, enterpriseId, storeCode, regionId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        StringBuilder storeIds = new StringBuilder();
        if (StringUtils.isBlank(storeCode)) {
            //查询域下所有的门店:q
//            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
//            storeSearchDTO.setEnterpriseId(enterpriseId);
//            storeSearchDTO.setRegionId(regionId);
//            ServiceResponse<Page<StoreDTO>> storeResult = storeApiService.listStore(storeSearchDTO, 0, Integer.MAX_VALUE);
//            if (storeResult.isSuccess()) {
//                Page<StoreDTO> pageStore = storeResult.getResult();
//                if (pageStore != null) {
//                    List<StoreDTO> storeList = pageStore.getResult();
//                    if (CollectionUtils.isNotEmpty(storeList)) {
//                        for (StoreDTO storeDTO : storeList) {
//                            storeIds.append(storeDTO.getStoreId()).append(",");
//                        }
//                    }
//                }
//            }
            //门店域有权限，但是域下面的门店不一定都有权限，进行过滤
            storeIds = getStoreIds(appId, enterpriseId, regionId);
            if (StringUtils.isBlank(storeIds.toString())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
            }
        } else {
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            storeIds.append(store.getStoreId());
        }
        //查询门店下所有的导购
        ClerkSearchDTO clerkSearchDTO = new ClerkSearchDTO();
        clerkSearchDTO.setPageSize(pageSize);
        clerkSearchDTO.setCurrentPage(pageNum);
        clerkSearchDTO.setStoreIds(storeIds.toString());
        ServiceResponse<Page<ClerkStoreListDTO>> clerkResult = clerkApiService.listClerk(enterpriseId, clerkSearchDTO, pageNum, pageSize);
        if (clerkResult.isSuccess()) {
            return ServiceResponse.success(JSON.toJSONString(clerkResult));
        }
        return ServiceResponse.failure(clerkResult.getCode(), clerkResult.getMessage());
    }

    /**
     * 验证权限门店相关权限
     * @param appId
     * @param enterpriseId
     * @param storeCode 门店code
     * @param regionId 门店域ID
     * @return
     */
    private boolean hasAuth(String appId, Integer enterpriseId, String storeCode, Integer regionId) {
        Map<String, Object> storeMap = new HashMap<>(16);
        Map<String, Object> regionMap = new HashMap<>(16);
        List<StoreDTO> list = getAuthByAppId(appId, enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (StoreDTO storeDTO : list) {
                storeMap.put(storeDTO.getStoreCode(), 1);
                regionMap.put(storeDTO.getRegionId().toString(), 1);
            }
        }
        if (StringUtils.isNotBlank(storeCode)) {
            if (storeMap.containsKey(storeCode)) {
                return true;
            }
        }
        if (regionId != null) {
            if (regionMap.containsKey(regionId.toString())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取有权限的门店域的有权限的所有门店，前提是门店域有权限
     * @param appId
     * @param enterpriseId
     * @param regionId
     * @return
     */
    private StringBuilder getStoreIds(String appId, Integer enterpriseId, Integer regionId) {
        StringBuilder storeIds = new StringBuilder();
        List<StoreDTO> list = getAuthByAppId(appId, enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (StoreDTO storeDTO : list) {
                if (storeDTO.getRegionId().intValue() == regionId.intValue()) {
                    storeIds.append(storeDTO.getStoreId()).append(" ");
                }
            }
        }
        return storeIds;
    }

    /**
     * 获取资源组门店资源
     * @param appId
     * @param enterpriseId
     * @return
     */
    private List<StoreDTO> getAuthByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<ApplicationResourceDTO> res = authorizeService
                .findResourceByEnterpriseIdAndAppId(Long.valueOf(enterpriseId), appId);
        if (res.isSuccess()) {
            ApplicationResourceDTO resourceDTO = res.getResult();
            if (resourceDTO != null) {
                Integer resourceId = resourceDTO.getResourceId();
                if (resourceId == null) {
                    return Collections.emptyList();
                }
                ServiceResponse<ResourceDTO> resourceResult = resourceApiService.getResource(resourceId);
                if (resourceResult.isSuccess()) {
                    LOGGER.info("对外API权限校验结果-门店资源ID:{}", JSON.toJSONString(resourceResult));
                    Long storeResourceId = resourceResult.getResult().getStoreResource();
                    //查询门店资源组所有的门店权限
                    ServiceResponse<Page<StoreDTO>> storeResult = storeWidgetApiService
                            .listStoreByStoreWidgetId(enterpriseId, storeResourceId.intValue(), 1, 10000);
                    if (storeResult.isSuccess()) {
                        Page<StoreDTO> page = storeResult.getResult();
                        if (page == null) {
                            return Collections.emptyList();
                        }
                        List<StoreDTO> list = page.getResult();
                        if (CollectionUtils.isNotEmpty(list)) {
                            return list;
                        }
                    } else {
                        LOGGER.info("对外API权限校验结果-门店资源获取:{}", JSON.toJSONString(storeResult));
                    }
                }
            }
            LOGGER.info("对外API权限校验结果-资源ID:{}", JSON.toJSONString(res));
        } else {
            LOGGER.info("对外API权限校验结果:{}", JSON.toJSONString(res));
        }
        return Collections.emptyList();
    }
}
